@@ -2229,21 +2229,25 @@ int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 	g_free(key);
 
 	key = g_strdup_printf("%slocal_address", prefix);
+	g_free(ipconfig->address->local);
 	ipconfig->address->local = g_key_file_get_string(
 			keyfile, identifier, key, NULL);
 	g_free(key);
 
 	key = g_strdup_printf("%speer_address", prefix);
+	g_free(ipconfig->address->peer);
 	ipconfig->address->peer = g_key_file_get_string(
 				keyfile, identifier, key, NULL);
 	g_free(key);
 
 	key = g_strdup_printf("%sbroadcast_address", prefix);
+	g_free(ipconfig->address->broadcast);
 	ipconfig->address->broadcast = g_key_file_get_string(
 				keyfile, identifier, key, NULL);
 	g_free(key);
 
 	key = g_strdup_printf("%sgateway", prefix);
+	g_free(ipconfig->address->gateway);
 	ipconfig->address->gateway = g_key_file_get_string(
 				keyfile, identifier, key, NULL);
 	g_free(key);
@@ -2265,13 +2269,15 @@ int __connman_ipconfig_save(struct connman_ipconfig *ipconfig,
 	const char *method;
 	char *key;
 
-	DBG("ipconfig %p identifier %s", ipconfig, identifier);
-
 	method = __connman_ipconfig_method2string(ipconfig->method);
 
-	key = g_strdup_printf("%smethod", prefix);
-	g_key_file_set_string(keyfile, identifier, key, method);
-	g_free(key);
+	DBG("ipconfig %p identifier %s method %s", ipconfig, identifier,
+								method);
+	if (method) {
+		key = g_strdup_printf("%smethod", prefix);
+		g_key_file_set_string(keyfile, identifier, key, method);
+		g_free(key);
+	}
 
 	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
 		const char *privacy;
